@@ -37,6 +37,11 @@ def default_config(tmpdir):
     return str(config_path)
 
 
+@pytest.fixture
+def tag_name():
+    return "v1.0.0"
+
+
 @pytest.fixture
 def group(gl):
     return gl.groups.get(1, lazy=True)
@@ -47,6 +52,21 @@ def project(gl):
     return gl.projects.get(1, lazy=True)
 
 
+@pytest.fixture
+def project_issue(project):
+    return project.issues.get(1, lazy=True)
+
+
+@pytest.fixture
+def project_merge_request(project):
+    return project.mergerequests.get(1, lazy=True)
+
+
+@pytest.fixture
+def release(project, tag_name):
+    return project.releases.get(tag_name, lazy=True)
+
+
 @pytest.fixture
 def user(gl):
     return gl.users.get(1, lazy=True)
